@@ -11,7 +11,7 @@ use std::convert::TryInto;
 use std::marker::PhantomData;
 use std::sync::{atomic::Ordering, Arc};
 use std::time::Duration;
-use tokio::io::AsyncWriteExt;
+use tokio::io::{AsyncWriteExt, BufWriter};
 use tokio::sync::mpsc::{unbounded_channel, UnboundedSender};
 use tokio::task::{yield_now, JoinHandle};
 use tokio::time::{sleep, timeout};
@@ -231,7 +231,8 @@ where
         Error::DosProtection
     }
 
-    async fn write(mut writer: WriteHalf<S>, mut rx: UnboundedReceiver<WriterMessage>) {
+    async fn write(writer: WriteHalf<S>, mut rx: UnboundedReceiver<WriterMessage>) {
+        let mut writer = BufWriter::new(writer);
         #[cfg(feature = "flatbuffers_helpers")]
         let mut builder = flatbuffers::FlatBufferBuilder::new();
         while let Some(message) = rx.recv().await {
@@ -241,7 +242,7 @@ where
                     sleep(duration).await;
                     break;
                 }
-                WriterMessage::Bytes(data) => {
+                WriterMessage::Send(data, flush) => {
                     if data.is_empty() {
                         yield_now().await;
                         continue;
@@ -250,6 +251,12 @@ where
                     if writer.write_all(&data).await.is_err() {
                         break;
                     }
+
+                    if flush {
+                        if let Err(error) = writer.flush().await {
+                            log::error!("Error when flushing {:?}", error);
+                        }
+                    }
                 }
                 #[cfg(feature = "flatbuffers_helpers")]
                 WriterMessage::SendFlatbuffers(f) => {
@@ -260,6 +267,11 @@ where
 
                     builder.reset();
                 }
+                WriterMessage::Flush => {
+                    if let Err(error) = writer.flush().await {
+                        log::error!("Error when flushing {:?}", error);
+                    }
+                }
             }
 
             yield_now().await;
@@ -282,8 +294,9 @@ fn parse_ping(content: Vec<u8>) -> Result<i64> {
 
 pub(crate) enum WriterMessage {
     Close,
+    Flush,
     CloseDelayed(Duration),
-    Bytes(Bytes),
+    Send(Bytes, bool),
     #[cfg(feature = "flatbuffers_helpers")]
     SendFlatbuffers(Box<dyn Fn(&mut FlatBufferBuilder<'static>) -> Bytes + Send + Sync>),
 }
